@@ -1,18 +1,21 @@
 """Caching decorators"""
 import inspect
+import logging
 
+from decorator import decorator
 from paste.deploy.converters import asbool
 
 import pylons
-from pylons.decorator import decorator
 
-def beaker_cache(key="cache_default", expire="never", type="dbm", 
-    query_args=False):
+log = logging.getLogger(__name__)
+
+def beaker_cache(key="cache_default", expire="never", type="dbm",
+    query_args=False, **b_kwargs):
     """Cache decorator utilizing Beaker. Caches action or other function that
     returns a pickle-able object as a result.
-    
+
     Optional arguments:
-    
+
     key
         None - No variable key, uses function name as key
         "cache_default" - Uses all function arguments as the key
@@ -25,31 +28,51 @@ def beaker_cache(key="cache_default", expire="never", type="dbm",
     query_args
         Uses the query arguments as the key, defaults to False
 
-    If cache_enabled is set to False in the .ini file, then cache is disabled 
+    If cache_enabled is set to False in the .ini file, then cache is disabled
     globally.
     """
     def wrapper(func, *args, **kwargs):
         """Decorator wrapper"""
-        enabled = pylons.g.pylons_config.app_conf.get("cache_enabled", "True")
+        log.debug("Wrapped with key: %s, expire: %s, type: %s, query_args: %s",
+                  key, expire, type, query_args)
+        enabled = pylons.config.get("cache_enabled", "True")
         if not asbool(enabled):
+            log.debug("Caching disabled, skipping cache lookup")
             return func(*args, **kwargs)
-        
-        my_cache = pylons.cache.get_cache(func.__module__ + "." + func.__name__)
+
+        my_cache = pylons.cache.get_cache('%s.%s' % (func.__module__,
+                                                     func.__name__))
         cache_key = _make_key(func, key, args, kwargs, query_args)
-        
+
         if expire == "never":
             cache_expire = None
         else:
             cache_expire = expire
         
-        content = my_cache.get_value(cache_key,
-            createfunc=lambda: func(*args, **kwargs), type=type,
-            expiretime=cache_expire)
-        return content
+        
+        def create_func():
+            log.debug("Creating new cache copy with key: %s, type: %s",
+                      cache_key, type)
+            result = func(*args, **kwargs)
+            glob_response = pylons.response._current_obj()
+            full_response = dict(headers=glob_response.headers,
+                                 status=glob_response.status_code,
+                                 cookies=glob_response.cookies,
+                                 content=result)
+            return full_response
+        
+        response = my_cache.get_value(cache_key, createfunc=create_func,
+                                     type=type, expiretime=cache_expire,
+                                     **b_kwargs)
+        glob_response = pylons.response._current_obj()
+        glob_response.headers = response['headers']
+        glob_response.status_code = response['status']
+        glob_response.cookies = response['cookies']
+        return response['content']
     return decorator(wrapper)
 
 def _make_key(func, key, args, kwargs, query_args):
-    """Helps make unique key from largs, kewargs and request.GET"""
+    """Helps make unique key from largs, kwargs and request.GET"""
     if key == "cache_default":
         if query_args:
             cache_key = repr(dict(pylons.request.GET))
